﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 설정 폼
    /// </summary>
    public partial class SettingForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 메인 폼
        /// </summary>
        private MainForm mainForm = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SettingForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public SettingForm()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Event
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 확인 버튼 클릭시 처리하기 - okButton_Click(sender, e)

        /// <summary>
        /// 확인 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void okButton_Click(object sender, EventArgs e)
        {
            this.mainForm.MaximumIterationCount = int.Parse(this.maximumIterationCountTextBox.Text);

            this.mainForm.Z1Real      = double.Parse(this.z1RealTextBox.Text     );
            this.mainForm.Z1Imaginary = double.Parse(this.z1ImaginaryTextBox.Text);
            this.mainForm.Z2Real      = double.Parse(this.z2RealTextBox.Text     );
            this.mainForm.Z2Imaginary = double.Parse(this.z2ImaginaryTextBox.Text);

            this.mainForm.ResetColorList();

            for(int i = 0; i < 48; i++)
            {
                Control control = GetControlByName(string.Format("color{0:d2}PictureBox", i));

                PictureBox pictureBox = control as PictureBox;

                if(pictureBox.BorderStyle == BorderStyle.Fixed3D)
                {
                    this.mainForm.ColorList.Add(pictureBox.BackColor);
                }
            }

            Close();
        }

        #endregion
        #region 취소 버튼 클릭시 처리하기 - cancelButton_Click(sender, e)

        /// <summary>
        /// 취소 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void cancelButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        #endregion

        #region 컬럼 버튼 클릭시 처리하기 - columnButton_Click(sender, e)

        /// <summary>
        /// 컬럼 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void columnButton_Click(object sender, EventArgs e)
        {
            Button button = sender as Button;

            SelectColumn(int.Parse(button.Tag.ToString()));
        }

        #endregion
        #region 행 버튼 클릭시 처리하기 - rowButton_Click(sender, e)

        /// <summary>
        /// 행 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void rowButton_Click(object sender, EventArgs e)
        {
            Button button = sender as Button;

            SelectRow(int.Parse(button.Tag.ToString()));        
        }

        #endregion
        #region 색상 픽처 박스 클릭시 처리하기 - colorPictureBox_Click(sender, e)

        /// <summary>
        /// 색상 픽처 박스 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void colorPictureBox_Click(object sender, EventArgs e)
        {
            PictureBox pictureBox = sender as PictureBox;

            if(pictureBox.BorderStyle == BorderStyle.Fixed3D)
            {
                pictureBox.BorderStyle = BorderStyle.None;
            }
            else
            {
                pictureBox.BorderStyle = BorderStyle.Fixed3D;
            }
        }

        #endregion

        #region 디폴트 버튼 클릭시 처리하기 - defaultButton_Click(sender, e)

        /// <summary>
        /// 디폴트 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void defaultButton_Click(object sender, EventArgs e)
        {
            this.noneButton.PerformClick();

            this.color40PictureBox.BorderStyle = BorderStyle.Fixed3D;
            this.color17PictureBox.BorderStyle = BorderStyle.Fixed3D;
            this.color18PictureBox.BorderStyle = BorderStyle.Fixed3D;
            this.color19PictureBox.BorderStyle = BorderStyle.Fixed3D;
            this.color20PictureBox.BorderStyle = BorderStyle.Fixed3D;
            this.color21PictureBox.BorderStyle = BorderStyle.Fixed3D;
            this.color22PictureBox.BorderStyle = BorderStyle.Fixed3D;
            this.color23PictureBox.BorderStyle = BorderStyle.Fixed3D;
        }

        #endregion
        #region 전체 선택 버튼 클릭시 처리하기 - allButton_Click(sender, e)

        /// <summary>
        /// 전체 선택 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void allButton_Click(object sender, EventArgs e)
        {
            foreach(Control control in Controls)
            {
                if(control is PictureBox)
                {
                    PictureBox pictureBox = control as PictureBox;

                    pictureBox.BorderStyle = BorderStyle.Fixed3D;
                }
            }
        }

        #endregion
        #region 전체 취소 버튼 클릭시 처리하기 - noneButton_Click(sender, e)

        /// <summary>
        /// 전체 취소 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void noneButton_Click(object sender, EventArgs e)
        {
            foreach(Control control in Controls)
            {
                if(control is PictureBox)
                {
                    PictureBox pictureBox = control as PictureBox;

                    pictureBox.BorderStyle = BorderStyle.None;
                }
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Function
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 색상 설정하기 - SelectColor(color)

        /// <summary>
        /// 색상 설정하기
        /// </summary>
        /// <param name="color">색상</param>
        private void SelectColor(Color color)
        {
            foreach(Control control in Controls)
            {
                if(control is PictureBox)
                {
                    if(control.BackColor.Equals(color))
                    {
                        PictureBox pictureBox = control as PictureBox;

                        pictureBox.BorderStyle = BorderStyle.Fixed3D;

                        return;
                    }
                }
            }
        }

        #endregion
        #region 초기화 하기 - Initialize(mainForm)

        /// <summary>
        /// 초기화 하기
        /// </summary>
        /// <param name="mainForm">메인 폼</param>
        public void Initialize(MainForm mainForm)
        {
            this.mainForm = mainForm;

            this.maximumIterationCountTextBox.Text = this.mainForm.MaximumIterationCount.ToString("0");
            this.z1RealTextBox.Text                = this.mainForm.Z1Real.ToString("0.00");
            this.z1ImaginaryTextBox.Text           = this.mainForm.Z1Imaginary.ToString("0.00");
            this.z2RealTextBox.Text                = this.mainForm.Z2Real.ToString("0.00");
            this.z2ImaginaryTextBox.Text           = this.mainForm.Z2Imaginary.ToString("0.00");

            this.noneButton.PerformClick();

            for(int i = 0; i < this.mainForm.ColorList.Count; i++)
            {
                SelectColor(this.mainForm.ColorList[i]);
            }
        }

        #endregion
        #region 명칭으로 컨트롤 구하기 - GetControlByName(name)

        /// <summary>
        /// 명칭으로 컨트롤 구하기
        /// </summary>
        /// <param name="name">명칭</param>
        /// <returns>컨트롤</returns>
        private Control GetControlByName(string name)
        {
            foreach(Control control in Controls)
            {
                if(control.Name == name)
                {
                    return control;
                }
            }

            return null;
        }

        #endregion
        #region 픽처 박스 선택하기 - SelectPictureBox(name)

        /// <summary>
        /// 픽처 박스 선택하기
        /// </summary>
        /// <param name="name">명칭</param>
        private void SelectPictureBox(string name)
        {
            Control control = GetControlByName(name);

            PictureBox pictureBox = control as PictureBox;

            pictureBox.BorderStyle = BorderStyle.Fixed3D;
        }

        #endregion
        #region 컬럼 선택하기 - SelectColumn(index)

        /// <summary>
        /// 컬럼 선택하기
        /// </summary>
        /// <param name="index">인덱스</param>
        private void SelectColumn(int index)
        {
            this.noneButton.PerformClick();

            string pictureBoxName;

            for(int i = 0; i <= 5; i++)
            {
                pictureBoxName = string.Format("color{0:d2}PictureBox", (i * 8 + index));

                SelectPictureBox(pictureBoxName);
            }
        }

        #endregion
        #region 행 선택하기 - SelectRow(index)

        /// <summary>
        /// 행 선택하기
        /// </summary>
        /// <param name="index">인덱스</param>
        private void SelectRow(int index)
        {
            this.noneButton.PerformClick();

            string pictureBoxName;

            int firstNumber = index * 8;

            for(int i = 0; i <= 7; i++)
            {
                pictureBoxName = string.Format("color{0:d2}PictureBox", (i + firstNumber));

                SelectPictureBox(pictureBoxName);
            }
        }

        #endregion
    }
}